<?php 

namespace Teknisa\CNC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DB\StorageProcedure\Param;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\CNC\Services\Cnc00300ProdutoAdquirenteService;
use Teknisa\CNC\Utilities;

use Teknisa\Libs\Util\Environment;

class Cnc00300ProdutoAdquirente extends \Teknisa\CNC\Controllers\TeknisaCrud {
    protected $dataSourceName = 'cnc00300_produtoadquirente';
    protected $parameterBagColumns;
    protected $produtoService;
    protected $environment;
    protected $salesMessage;
    
    public function __construct($crudService, $parameterBag, $environment, Cnc00300ProdutoAdquirenteService $produtoService, Utilities\SalesMessageCNC $salesMessage, Utilities\Util $util) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment = $environment;
        $this->produtoService = $produtoService;
        $this->salesMessage   = $salesMessage;
        $this->util = $util;
    }
    
    public function getProdAdq(Request\Filter $request, Response $response){
        try {
            $row['NRORG'] = $this->environment->getNrOrgTrab();
            $Result = $this->produtoService->getProdAdq($row);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProdutoAdquirenteRepo', $Result));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function getTipoRecebimento(Request\Filter $request, Response $response){
        try {
            $this->dataSourceName = 'cnc00300_produtoadquirente_tiposderecebimento';
            $this->find($request, $response);
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function getTipoRecebimentoSelect(Request\Filter $request, Response $response) {
        try {
            $this->dataSourceName = 'cnc00300_produtoadquirente_tiposderecebimento_tipos';
            $this->find($request, $response);
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function deleteTipoRece (DTO\Request $request, DTO\Response $response){
        try {
            $this->dataSourceName = 'cnc00300_produtoadquirente_tiposderecebimento';
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function saveTipoRece (DTO\Request $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $codigo = $this->produtoService->getNovoCodigoTipoRece($row);
            $row->set('NRTPRECEPRODADQ', $codigo);
            $this->dataSourceName = 'cnc00300_produtoadquirente_tiposderecebimento';
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function getProdutosGer(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params['NRORG'] = $this->environment->getNrOrgTrab();
            $Result = $this->produtoService->getProdutosGer($params);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProdutosGerRepo', $Result));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function getProdutos(Request\Filter $request, Response $response){
        $allFieldsLabel = 'NMPRODUTOADQ|VRTOLERANCIA|LBIDVISIVEL';
        
        try {
            $filterConditions = $request->getFilterCriteria()->getConditions();
            
            $params['NRORG']        = $this->environment->getNrOrgTrab();
            $params['NRPRODUTOADQ'] = $this->getParamValueFromConditions($filterConditions, "NMPRODUTOADQ");
            $params['CDTIPORECE']   = $this->getParamValueFromConditions($filterConditions, "NMTIPORECE");
            
            $params['TNRPTODUTOADQ'] = null;
            if (!is_array($params['NRPRODUTOADQ']))
                $params['TNRPTODUTOADQ'] = "T";
            else 
                $params['NRPRODUTOADQ'] = $this->buildParamValue($params['NRPRODUTOADQ']);
            
            $params['TCDTIPORECE'] = null;
            if (!is_array($params['CDTIPORECE']))
                $params['TCDTIPORECE'] = "T";
            else
                $params['CDTIPORECE'] = $this->buildParamValue($params['CDTIPORECE']);
            
            $params['ALL_FIELDS'] = $this->getParamValueFromConditions($filterConditions, $allFieldsLabel);

            if(empty($params['ALL_FIELDS'])) {
                $params['ALL_FIELDS'] = '';
                $params['TALL_FIELDS'] = 'T';
            } else {
                $params['ALL_FIELDS'] = str_replace('%', '', $params['ALL_FIELDS'][0]['_ALL']);
                $params['TALL_FIELDS'] = 'F';
            }       

            $result = $this->produtoService->getProdutos($params);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cnc00300_produtoadquirente', $result));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
	public function save (DTO\Request $request, DTO\Response $response) {
	    try {
    	    $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            foreach ($rows as $row) {
                
                if (is_object($row))
                    $row = (array) $row;
                
                $row['NRORG'] = $this->environment->getNrOrgTrab();
                $row['IDATIVO'] = isset($row['IDATIVO']) ? ($row['IDATIVO'] ?: 'S') : 'S';
                $row['DTULTATU'] = $currentTime;
                $row['NRORGULTATU'] = $this->environment->getNrOrgTrab();
                $row['CDOPERULTATU'] = $this->environment->getCdOperador();
                $this->produtoService->saveProduto($row);
                $response->addNotification(new Notification("Dados salvos com sucesso!"), Notification::TYPE_SUCCESS);
            }
	    } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
	}
	
	public function saveAdq (DTO\Request $request, DTO\Response $response) {
	    try {
	        $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            
            foreach ($rows as $row) {
                if (is_object($row))
                    $row = (array) $row;
                
                $row['NRORG'] = $this->environment->getNrOrgTrab();
                $row['IDATIVO'] = isset($row['IDATIVO']) ? ($row['IDATIVO'] ?: 'S') : 'S';
                $row['DTULTATU'] = $currentTime;
                $row['NRORGULTATU'] = $this->environment->getNrOrgTrab();
                $row['CDOPERULTATU'] = $this->environment->getCdOperador();
                $vrTaxa = $this->produtoService->saveAdquirente($row);
                
                $response->addNotification(new Notification("Dados salvos com sucesso!|".$vrTaxa), Notification::TYPE_SUCCESS);
            }
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}
	
	public function saveEstabelec (DTO\Request $request, DTO\Response $response) {
	    try {
	        $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            foreach ($rows as $row) {
                
                if (is_object($row))
                    $row = (array) $row;
                
                $row['hasEstabelec'] = isset($row['NRESTABELEC']);
                
                $row['NRORG'] = $this->environment->getNrOrgTrab();
                $row['IDATIVO'] = isset($row['IDATIVO']) ? ($row['IDATIVO'] ?: 'S') : 'S';
                $row['DTULTATU'] = $currentTime;
                $row['NRORGULTATU'] = $this->environment->getNrOrgTrab();
                $row['CDOPERULTATU'] = $this->environment->getCdOperador();
                $this->produtoService->saveEstabelec($row);
                $response->addNotification(new Notification("Dados salvos com sucesso!"), Notification::TYPE_SUCCESS);
            }
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}
	
	public function saveEditEstabelec (DTO\Request $request, DTO\Response $response) {
	    try {
	        $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            
            foreach ($rows as $row) {
                
                if (is_object($row))
                    $row = (array) $row;
                
                $row['NRORG'] = $this->environment->getNrOrgTrab();
                $row['IDATIVO'] = isset($row['IDATIVO']) ? ($row['IDATIVO'] ?: 'S') : 'S';
                $row['DTULTATU'] = $currentTime;
                $row['NRORGULTATU'] = $this->environment->getNrOrgTrab();
                $row['CDOPERULTATU'] = $this->environment->getCdOperador();
                
                $this->produtoService->saveEditEstabelec($row);
                $response->addNotification(new Notification("Dados salvos com sucesso!"), Notification::TYPE_SUCCESS);
            }
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}
	
	public function removeEstabelec (DTO\Request $request, DTO\Response $response) {
	    try {
	        $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();
            
	        foreach ($rows as $row) {
	            
	            if (is_object($row))
                    $row = (array) $row;
                    
                $row['NRORG'] = $this->environment->getNrOrgTrab();
                $row['IDATIVO'] = isset($row['IDATIVO']) ? ($row['IDATIVO'] ?: 'S') : 'S';
                
                $this->produtoService->removeEstabelec($row);
                $response->addNotification(new Notification("Dados apagados com sucesso!"), Notification::TYPE_SUCCESS);
	        }
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}

    public function getPeriodTaxa(DTO\Request $request, DTO\Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            if(isset($params[3])){
                $resultado = $this->produtoService->getPeriodTaxaRow($params);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc00300_adquirentePeriodoRow', $resultado));
            }else{
                $resultado = $this->produtoService->getPeriodTaxa($params);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cnc00300_adquirentePeriodos', $resultado));
            }


        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function savePeriodoTaxa(DTO\Request $request, DTO\Response $response) {
        try {
            $params = $request->getDataSet()->getRows();
            if($params[0]["__is_new"] == true){
                $this->produtoService->inserirPeriodoTaxa($params);
                $message= array($this->salesMessage->getMessage('criado'));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc00300_InserePeriodoTaxaRepo', $message));
            }else{
                $this->produtoService->alterarPeriodoTaxa($params);
                $message= array($this->salesMessage->getMessage('alterado'));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc00300_AlteraPeriodoTaxaRepo', $message));
            }

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function alterarPeriodoTaxa(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $this->produtoService->alterarPeriodoTaxa($params);
            
            $message= array($this->salesMessage->getMessage('alterado'));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc00300_AlteraPeriodoTaxaRepo', $message));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function inserirPeriodoTaxa(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $this->produtoService->inserirPeriodoTaxa($params);
            
            $message= array($this->salesMessage->getMessage('criado'));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc00300_InserePeriodoTaxaRepo', $message));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function deletarPeriodoTaxa(DTO\Request $request, DTO\Response $response) {
        try {
            //$params = $request->getFilterCriteria()->getConditions();
            $dataSet = $request->getDataSet();
            $params = $dataSet->getRows();
            $this->produtoService->deletarPeriodoTaxa($params);
            
            $message= array($this->salesMessage->getMessage('excluido'));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc00300_DeletaPeriodoTaxaRepo', $message));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function getParamValueFromConditions ($conditions, $paramName) {
        $arr = array_filter($conditions, function($param) use ($paramName) {
            return $param["columnName"] === $paramName;
        });
        
        return array_shift($arr)["value"];
    }
    
    public function buildParamValue ($arr) {
        return array_reduce($arr, function($accumulator, $element) {
            return $accumulator."|".$element;
        }, "")."|";
    }
    
    public function getTaxas(Request\Filter $request, Response $response) {
        try {
           $this->dataSourceName = 'cnc00300_taxaPorParcela';
            parent::find($request, $response);
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function saveTaxa(DTO\Request $request, DTO\Response $response) {
        try {
           $this->dataSourceName = 'cnc00300_taxaPorParcela';
           $conditions = $request->getDataSet()->getRows()[0];
           $conditions['NRTAXAADQPARC'] = intval($this->util->newCode('TAXAADQPARC', 20));
           $conditions['VRTAXAPARC'] = floatval(str_replace(',', '.', $conditions['VRTAXAPARC']));
           $registerExists = $this->produtoService->registerExists($conditions);
           if ($registerExists && $conditions['__is_new']) {
               $response->addMessage(new Message('Registro já existe.'), Message::TYPE_ERROR);
            } 
           else if($conditions['VRTAXAPARC'] <= 0){
               $response->addMessage(new Message('O valor da taxa por parcela deve ser maior que 0.'), Message::TYPE_WARNING);
           }
           else if($conditions['QTPARCELA'] > 999){
               $response->addMessage(new Message('O máximo de parcelas permitidas são 999.'), Message::TYPE_WARNING);
           }
           else if($conditions['QTPARCELA'] <= 0){
               $response->addMessage(new Message('O número de parcelas deve ser maior que 0.'), Message::TYPE_WARNING);
           } 
           else if($conditions['VRTAXAPARC'] >= 100){
               $response->addMessage(new Message('A taxa máxima por parcela é de 100%.'), Message::TYPE_WARNING);
           }
            else {
                parent::save($request, $response);
           }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function deleteTaxa (DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'cnc00300_taxaPorParcela';
        parent::delete($request, $response);
    }
}