<?php 

namespace Teknisa\CNC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

use Teknisa\CNC\Utilities;

use Teknisa\CNC\Services\Cnc00500VendaPorAdquirenteService;

class Cnc00500VendaPorAdquirente {
	protected $parameterBagColumns;
	protected $util;
	protected $vendaService;
	
	public function __construct(ParameterBag $parameterBag, Utilities\Util $util, Cnc00500VendaPorAdquirenteService $vendaService) {
        $this->parameterBag = $parameterBag;
        $this->util = $util;
        $this->vendaService = $vendaService;
	}
	
	public function find (DTO\Request\Row $request, DTO\Response $response) {
	    try {
	        $row = $request->getRow();
	        if($row["NRESTABELEC"] != ""){
		        $row["NRESTABELEC"] = array_reduce($row["NRESTABELEC"], function($acc, $ele) {
		        	return $acc."|".$ele;
		        }, "")."|";
	        } else{
	        	$row["NRESTABELEC"] = 'T';
	        }
	        
	        if($row["NRADQUIRENTE"] != ""){
	        $row["NRADQUIRENTE"] = array_reduce($row["NRADQUIRENTE"], function($acc, $ele) {
	        	return $acc."|".$ele;
	        }, "")."|";
	        } else{
	        	$row["NRADQUIRENTE"] = 'T';
	        }
            $dataSet = $this->vendaService->getVendasPorAdquirente($row);
            $response->addDataSet(new DataSet('cnc00500_vendaporadquirente', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
	}
	
	public function selectDetails (DTO\Request\Filter $request, DTO\Response $response) {
	    try {
    	    $filter = $request->getFilterCriteria()->getConditions();
    	    
    	    $filter["NRESTABELEC"] = array_filter($filter, function($param) {
	            return $param["columnName"] === "NRESTABELEC";
	        });
	        $filter["NRESTABELEC"] = array_shift($filter["NRESTABELEC"]);
	        
	        $filter["NRADQUIRENTE"] = array_filter($filter, function($param) {
	            return $param["columnName"] === "NRADQUIRENTE";
	        });
	        $filter["NRADQUIRENTE"] = array_shift($filter["NRADQUIRENTE"]);
	        
	        $filter["DTVENDA"] = array_filter($filter, function($param) {
	            return $param["columnName"] === "DTVENDA";
	        });
	        $filter["DTVENDA"] = array_shift($filter["DTVENDA"]);
	        
	        $filter["TIPORECE"] = array_filter($filter, function($param) {
	            return $param["columnName"] === "TIPORECE";
	        });
	        $filter["TIPORECE"] = array_shift($filter["TIPORECE"]);
	        
	        $filter["TIPOVENDA"] = array_filter($filter, function($param) {
	            return $param["columnName"] === "TIPOVENDA";
	        });
	        $filter["TIPOVENDA"] = array_shift($filter["TIPOVENDA"]);
	        
	        if($filter["NRESTABELEC"]["value"] != ""){
		        $filter["NRESTABELEC"]["value"] = array_reduce($filter["NRESTABELEC"]["value"], function($acc, $ele) {
		        	return $acc."|".$ele;
		        }, "")."|";
	        } else{
	        	$filter["NRESTABELEC"]["value"] = 'T';
	        }
	        
	        if($filter["NRADQUIRENTE"]["value"] != ""){
		        $filter["NRADQUIRENTE"]["value"] = array_reduce($filter["NRADQUIRENTE"]["value"], function($acc, $ele) {
		        	return $acc."|".$ele;
		        }, "")."|";
	        } else{
	        	$filter["NRADQUIRENTE"]["value"] = 'T';
	        }
    	    $dataSet = $this->vendaService->selectDetails($filter);
    	    $response->addDataSet(new DataSet('cnc00500_details_vendas', $dataSet));
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}
}