<?php
namespace Teknisa\CNC\Services;

use Teknisa\CNC\Utilities;
use Teknisa\CNC\Factories;

class EstabelecimentoCNC {

    protected $entityManager;
    protected $util;
    protected $estabelecFactory;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Utilities\UtilCNC $util, Factories\CncEstabelecFactory $estabelecFactory)  {
        $this->entityManager    = $entityManager;
        $this->util             = $util;
        $this->estabelecFactory  = $estabelecFactory;
    }

    public function getEstabelecAdq ($params, $request) {
        $NRADQUIRENTE = $params[0]['value'];
        $NRORG = $params[1]['value'];  
        $P_TNRESTABELEC = null;
        $P_NRESTABELEC = null;
        for ($i=0; $i < sizeof($params); $i++) {
            if($params[$i]["columnName"] === 'LABELESTABELEC'){
                $P_NRESTABELEC = $params[$i]['value'];
            }
        }
        if($P_NRESTABELEC === null){
            $P_TNRESTABELEC = 'T';
        }
        $ficaFrioAi = null;
        if(isset($params[2])){
            if(is_array($params[2]['value'])){
                $ficaFrioAi = true;
                $P_NRESTABELEC = $params[2]['value'];
    
                if(isset($params[3])) {
                    $ALL_FIELDS = $params[3]['value'];
                    $TALL_FIELDS = 'N';
                } else {
                    $ALL_FIELDS = null;
                    $TALL_FIELDS = 'T';
                }
            
                $paramsfilial = array(
                    ':P_NRESTABELEC' => $P_NRESTABELEC
                );
                $query = "
                SELECT DISTINCT NRESTABELEC, CDESTABELEC, NMESTABELEC, CDESTABELEC||' - '||NMESTABELEC AS LABELESTABELEC, 
                    'Não' PARAMETRIZADO FROM ( 
                        SELECT DAT.NRESTABELEC, DAT.NMESTABELEC, DAT.CDESTABELEC, 
                               DAT.CDESTABELEC||' - '||DAT.NMESTABELEC AS LABELESTABELEC, N 
                          FROM ( SELECT D.*, ROWNUM as N 
                                   FROM ( SELECT NRESTABELEC, NMESTABELEC, CDESTABELEC, 
                                                 CDESTABELEC||' - '||NMESTABELEC AS LABELESTABELEC, NRORG 
                                            FROM ESTABELECIMENTO
                                           WHERE ((INSTR (':P_NRESTABELEC', TO_CHAR(NRESTABELEC)) <> 0) OR ('T' = ':P_TNRESTABELEC')) 
                                                 AND (( UPPER(CDESTABELEC||' - '||NMESTABELEC) LIKE '%'|| UPPER(:ALL_FIELDS) || '%' ) OR ( ':TALL_FIELDS' = 'T'))
                                          ) D ) DAT 
                                  WHERE DAT.NRORG = :NRORG ) F 
                         WHERE NOT EXISTS (SELECT 1 FROM VALIDACAOADQEST V 
                         WHERE V.NRADQUIRENTE = :NRADQUIRENTE AND V.NRORDEM IS NOT NULL 
                                AND V.NRESTABELEC = F.NRESTABELEC) UNION 
                SELECT V.NRESTABELEC, F.CDESTABELEC, F.NMESTABELEC, F.CDESTABELEC||' - '||F.NMESTABELEC AS LABELESTABELEC, 
                        'Sim' PARAMETRIZADO 
                  FROM VALIDACAOADQEST V 
                  JOIN( SELECT DAT.NRESTABELEC, DAT.NMESTABELEC, DAT.CDESTABELEC, DAT.CDESTABELEC||' - '||DAT.NMESTABELEC AS LABELESTABELEC, N 
                  FROM ( SELECT D.*, ROWNUM as N 
                  FROM ( SELECT NRESTABELEC, NMESTABELEC, CDESTABELEC, CDESTABELEC||' - '||NMESTABELEC AS LABELESTABELEC, 
                                NRORG 
                           FROM ESTABELECIMENTO 
                          WHERE ((INSTR (':P_NRESTABELEC', TO_CHAR(NRESTABELEC)) <> 0) OR ('T' = ':P_TNRESTABELEC')) 
                            AND (( UPPER(CDESTABELEC||' - '||NMESTABELEC) LIKE '%'|| UPPER(:ALL_FIELDS) || '%' ) OR ( ':TALL_FIELDS' = 'T'))
                          ORDER BY NRESTABELEC, LABELESTABELEC ) D ) DAT 
                    WHERE N BETWEEN :FIRST AND :LAST AND DAT.NRORG = :NRORG 
                    ORDER BY DAT.NRESTABELEC, DAT.NMESTABELEC, DAT.CDESTABELEC, LABELESTABELEC ) F 
                    ON V.NRESTABELEC = F.NRESTABELEC AND V.NRADQUIRENTE = :NRADQUIRENTE AND V.NRORDEM IS NOT NULL 
                        AND V.IDATIVO = 'S'"; 
                foreach (array_keys($paramsfilial) as $index => $name) {
                    if(!empty($paramsfilial[$name])){
                        $query = $this->util->putArrayOnQuery($query, $name, $paramsfilial[$name]);
                        $paramsfilial[$name] = '';
                    }
                    else{
                        $paramsfilial[$name] = 'null';
                    }
                }
                $query = preg_replace('/'.':NRADQUIRENTE'.'/', $NRADQUIRENTE, $query, 2);
                $query = preg_replace('/'.':NRORG'.'/', $NRORG, $query, 2);
                $query = preg_replace('/'.':LAST'.'/', 10000, $query);
                $query = preg_replace('/'.':FIRST'.'/', 0, $query);
                $params = array(
                    ':P_TNRESTABELEC' => ''
                );
                
                if (empty($P_NRESTABELEC)) {
                    $params[':P_NRESTABELEC'] = 'T';
                    $params[':P_TNRESTABELEC'] = 'T';
                } else {
                    $params[':P_NRESTABELEC'] =  implode('_', $P_NRESTABELEC);
                    $params[':P_TNRESTABELEC'] = 'F';
                }
                
                if (empty($ALL_FIELDS)) {
                    $params[':ALL_FIELDS'] = 'T';
                } else {
                    $params[':ALL_FIELDS'] = $ALL_FIELDS[0]['_ALL'];
                }

                $params[':TALL_FIELDS'] = $TALL_FIELDS;
                
                $query = preg_replace('/'.':TALL_FIELDS'.'/', $TALL_FIELDS, $query);
                $query = preg_replace('/'.':P_NRESTABELEC'.'/', $params[':P_NRESTABELEC'], $query);
                $query = preg_replace('/'.':P_TNRESTABELEC'.'/', $params[':P_TNRESTABELEC'], $query);
                
                $filters = array(
                    ':ALL_FIELDS' => $params[':ALL_FIELDS']    
                );

                return $this->estabelecFactory->getEstabelecAdqFiltered($filters, $query);
            }
            $P_NRESTABELEC  = $params[2]['value'];
        }else{
            $ficaFrioAi= false;
        }
        if(!$ficaFrioAi){
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $FIRST = $request->getFilterCriteria()->getPageSize() * ($page-1);
            $LAST = ($FIRST + $pageSize);
            if($page > 1){
                $FIRST++;
            }
            $params = array(
                ':FIRST'          => $FIRST,
                ':LAST'           => $LAST,
                ':P_NRESTABELEC'  => $P_NRESTABELEC,
                ':P_TNRESTABELEC' => $P_TNRESTABELEC,
                ':NRADQUIRENTE'   => $NRADQUIRENTE,
                ':NRORG'          => $NRORG
            );
            $estabelec = $this->estabelecFactory->getEstabelecAdq($params);
            return $estabelec;
        }
    }

    public function getEstabelec ($params, $request) {
        $NRORG = $params[0]['value'];

        $ficaFrioAi = null;
        
        $LABELESTABELEC = "";
            
        if(isset($params[1])){
            if(is_array($params[1]['value']) || (isset($params[2]) && $params[2]["columnName"] === 'CDESTABELEC|NMESTABELEC')){
                $ficaFrioAi = true;
                for ($i=0; $i < sizeof($params); $i++) {
                    if( ($params[$i]["columnName"] === 'CDESTABELEC|NMESTABELEC') ||
                        $params[$i]["columnName"] === 'LABELESTABELEC' || 
                        $params[$i]["columnName"] === 'NMESTABELEC' || 
                        strpos($params[$i]["columnName"], 'NMESTABELEC') == true) {
                        $LABELESTABELEC = $params[$i]['value'];
                    }
                }
                

                $paramsfilial = array(
                    ':LABELESTABELEC' => $LABELESTABELEC
                );
                $query = "
                    SELECT  DAT.NRESTABELEC, DAT.NMESTABELEC, DAT.CDESTABELEC, DAT.CDFILIAL, DAT.NMFILIAL,DAT.CDEMPRESA, 
                            DAT.CDINSCESTA, DAT.CDINSCMUNI, DAT.CDESTABELEC||' - '||DAT.NMESTABELEC AS LABELESTABELEC, 
                            DAT.CDFILIAL||' - '||DAT.NMFILIAL AS LABELFILIAL, N 
                      FROM (SELECT D.*, ROWNUM as N 
                              FROM (SELECT  E.NRESTABELEC, E.NMESTABELEC, E.CDESTABELEC, E.CDFILIAL, F.NMFILIAL, E.CDEMPRESA, 
                                            E.CDINSCESTA, E.CDINSCMUNI, E.CDESTABELEC||' - '||E.NMESTABELEC AS LABELESTABELEC, 
                                            E.NRORG 
                                      FROM ESTABELECIMENTO E 
                                      LEFT JOIN FILIAL F 
                                        ON E.CDFILIAL = F.CDFILIAL 
                                     WHERE UPPER (E.CDESTABELEC||' - '||E.NMESTABELEC) 
                                      LIKE UPPER (':LABELESTABELEC') AND E.IDATIVO = 'S' ) D ) DAT 
                             WHERE N BETWEEN :FIRST AND :LAST 
                               AND DAT.NRORG = :NRORG 
                             ORDER BY DAT.CDFILIAL, DAT.NRESTABELEC, DAT.CDESTABELEC, DAT.NMESTABELEC, LABELESTABELEC"; 
                foreach (array_keys($paramsfilial) as $index => $name) {
                    if(!empty($paramsfilial[$name])){
                        $query = $this->util->putArrayOnQuery($query, $name, $paramsfilial[$name]);
                        $paramsfilial[$name] = '';
                    }
                    else{
                        $paramsfilial[$name] = 'null';
                    }
                }
                $page = $request->getFilterCriteria()->getPage();
                $pageSize = $request->getFilterCriteria()->getPageSize();
                $FIRST = $request->getFilterCriteria()->getPageSize() * ($page-1);
                $LAST = ($FIRST + $pageSize);
                if($page > 1){
                    $FIRST++;
                }
                $query = preg_replace('/'.':NRORG'.'/', $NRORG, $query, 1);
                $query = preg_replace('/'.':FIRST'.'/', $FIRST, $query, 1);
                $query = preg_replace('/'.':LAST'.'/', $LAST, $query, 1);
                $resultadoFiltrado = $this->estabelecFactory->getEstabelecFiltered($query);
                return $resultadoFiltrado;
            }   
            for ($i=0; $i < sizeof($params); $i++) {
                if($params[$i]["columnName"] === 'LABELESTABELEC'){
                    $LABELESTABELEC = $params[$i]['value'];
                }
                if($LABELESTABELEC === ""){
                    $LABELESTABELEC = "%%";
                }
            }
            $LABELESTABELEC = $params[1]['value'];
        }else{
            $ficaFrioAi = false;
            $LABELESTABELEC = "%%";
        }
        if(!$ficaFrioAi){
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $FIRST = $request->getFilterCriteria()->getPageSize() * ($page-1);
            $LAST = ($FIRST + $pageSize);
            if($page > 1){
                $FIRST++;
            }
            $params = array(
                ':FIRST'            => $FIRST,
                ':LAST'             => $LAST,
                ':LABELESTABELEC'   => $LABELESTABELEC,
                ':NRORG'            => $NRORG
            );
            
            $filial = $this->estabelecFactory->getEstabelec($params);
            return $filial;
        }
    }

}